/***************************************************************************
                          foundationcore.cpp  -  description
                             -------------------
    begin                : Sun May 21 2000
    copyright            : (C) 2000 by Blake Friesen
    email                : umfrie64@cc.umanitoba.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <fstream.h>
#include "foundationcore.h"

FoundationCore::FoundationCore()
{
	loadShipSpecs();
	/* 	init planets
		should be changed to a user supplied var */
	for(int i = 0;i<PLANET_NUM;i++)
	{
		SolarSystem *s = new SolarSystem();
		s->setId( i );
		planet.push_back(s);
	}
}

FoundationCore::~FoundationCore(){
}

/** a major loop
waits for network packet and handles
accordingly
 */
void FoundationCore::net_wait()
{
	char *buf = (char *)malloc(MAX_PACKET_SIZE);
	memset(buf, 0, MAX_PACKET_SIZE);
	char type, funct;
	struct sockaddr_in addr;
	int len = ne.getpacket(&addr, &type, &funct, buf);
	if(len == 0) return;
	printf("net wait got: %d %d %d\n", len, type, funct);
	switch (type)
	{
		case(GAME_UPDATE):
			switch(funct)
			{
				case(GAME_JOIN):
				handle_game_join(addr, buf, len);
				break;
				case(GAME_START):
				handle_game_start(addr, buf, len);
				break;
				case(GAME_END):
				break;
				case(CHAT_MSG):
				handle_chat_msg(buf, len);
				break;
			}
		break;
		case(PLANET_UPDATE):
			switch(funct)
			{
				case(PRODUCTION_DONE):
					break;
				case(PRODUCTION_REQUEST):
					handle_prod_request(addr, buf, len);
					break;
				case(PLANET_UPDATE_PART):
					handle_planet_part(addr, buf, len);
					break;
				case(PLANET_UPDATE_BUILDING):
    				break;
				case(PLANET_UPDATE_OWNER):
    				break;
				case(PLANET_UPDATE_INFO):
	           		handle_planet_info(buf, len);
					break;
			}
		break;
		case(FLEET_UPDATE):
			switch(funct)
			{
				case(FLEET_ADD):
					handle_fleet_add(addr, buf);
					break;
				case(FLEET_ADD_WAYPOINT):
					printf("add waypoint for a fleet\n");
					handle_add_waypoint(addr, buf, len);
					break;
				case(FLEET_TRANSFER_CARGO):
					handle_transfer_cargo(addr, buf, len);		
					break;
			}
		break;
	}
	free (buf);
}

/** finds a planet by it's coordinates
(will do) does a binary search and assumes
that the planets are sorted
 */
int FoundationCore::find_planet(Point p)
{
	int i;
	for(i = 0;i<planet.size();i++)
	{
		if( planet[i]->pos.x == p.x &&
			planet[i]->pos.y == p.y &&
			planet[i]->pos.z == p.z)
			{

			printf("find planet returns %d\n", i);
			return i;
			}
	}
	printf("planet not found\n");
	return -1;
}

void FoundationCore::loadnames(){
        ifstream input("star_names.txt");
        string s;
        for(int x =0;x<PLANET_NUM;x++){
                input >> s;
                planet[x]->setName(s);
//              cout << "\nplanet name: " << planet[x].name;

                }
        input.close();
}

/* network handlers */

void FoundationCore::handle_chat_msg(char *buf, int len)
{
	printf("handle_chat_msg should not be called\n");
};

void FoundationCore::handle_game_join(sockaddr_in addr, char *buf, int len){
}

void FoundationCore::handle_game_start(sockaddr_in addr, char *buf, int len)
{
}

/** handles incoming updates about planet
changes.  Should only be used by the
client. */
void FoundationCore::handle_planet_info(char *buf, int len)
{
	/* this is a bug and should really be empty!!! */
	printf("should not be called\n");
	int i = 0;
	memcpy(&i, buf, sizeof(i));
//	printf("setting planet %d...\n", i);
	planet[i]->setRaw(buf+sizeof(i));
}
/**  */
void FoundationCore::handle_planet_part(sockaddr_in addr, char *buf, int len)
{
}

void FoundationCore::handle_prod_request(sockaddr_in addr, char *buf, int len)
{
	assert(0);
}

/** handles new fleet addition, meant to be
used by the client */

void FoundationCore::handle_fleet_add(sockaddr_in addr, char *buf)
{
	assert(0);
}

int FoundationCore::add_waypoint(Fleet *f, int p)
{
/* change or eliminate this */
	return setDest(f->getId(), planet[p]->getPos() );	
}

void FoundationCore::handle_add_waypoint(sockaddr_in addr, char *buf, int len)
{
}

void FoundationCore::handle_transfer_cargo(sockaddr_in addr, char *buf, int len)
{
}


/* added from shipinfo */


void FoundationCore::loadShipSpecs()
{
/* needs to be replaced by a proper lex parser */
	Ship s;
	int id, cargo, pass, guns, canons, tor, speed = 0;
	ifstream input("shipspecs.txt");
	for(int x=0;x<T_NUM;x++)
	{
		input >> id;
		input >> cargo;
		input >> guns;
		input >> canons;
		input >> tor;
		input >> speed;
		s.set(guns, canons, tor, pass, cargo);
		s.setType(x);
		shiptype_list.push_back(s);
	}
}

int FoundationCore::loadFleet(int f, int n, int t)
{
	return fleet[f]->loadCargo(n, t);
}

Fleet *FoundationCore::getFleet(int f)
{
	for(int i = 0;i<fleet.size();i++)
		if ((int)fleet[i]->getId()==f) return fleet[i];
	cerr << "FleetID reference out of range: " << fleet.size() << endl;
	return NULL;
}

int FoundationCore::addShip(int s, Point p, int owner)
{
	int fleet_id;
	Ship newship;
	newship = shiptype_list[s];
	cout << "adding ship type " << s << " for " << owner << endl;


/* check if there is an existing fleet at this position
   If there is, add it to a fleet */

	vector<Fleet *> fleet_here = getFleet( p );
	if( fleet_here.size() != 0)
	{
		fleet_here[0]->addShip( newship );
		fleet_id = fleet_here[0]->getId();
	}
/* otherwise, make a new fleet */
	else
	{
		printf("making new fleet\n");
		Fleet *f = new Fleet(p);
		f->addShip(newship);
		f->owner = owner;
		fleet.push_back(f);
		fleet_id = f->getId();
	}
	return fleet_id;
}

vector<Platform *> FoundationCore::getPlatform(Point p)
{
	int planet_found;
	vector<Platform *> platforms;
	vector<Fleet *> f;
	f = getFleet( p );

	for(int i = 0;i<f.size();i++)
	{
		platforms.push_back( f[i] );
	}

	planet_found = find_planet( p );
	if( planet_found != -1)
	{
		printf("found planet %d, is at %d",
			planet_found, planet[planet_found]);
		
		platforms.push_back( planet[planet_found] );
	}
	return platforms;
}


vector<Fleet *> FoundationCore::getFleet(Point p)
{
	vector<Fleet *> a;
	for(int x=0;x<(int)fleet.size();x++)
		if(fleet[x]->getPos()==p) a.push_back(fleet[x]);
	return a;
}

vector<Fleet *> FoundationCore::getFleetOwner(int owner)
{
	vector<Fleet *> a;
	printf("looking for fleets owned by %d\n", owner);
	for(int x = 0;x < (int)fleet.size();x++)
	{
		if( fleet[x]->owner == owner )
		{ 
			a.push_back(fleet[x]);
		}
	}
	printf("found %d fleets\n", a.size() );
	return a;
}

int FoundationCore::xfer(Platform *src, Platform *dest, int amount, int cargo_type)
{
	/*	Transfers up to N units, limited by free cargo space
		Then unloads the amount that was succesfully loaded */
	int cargo_amount, loaded;
	cargo_amount = src->loadCargo( -amount, cargo_type);
	loaded = dest->loadCargo(-cargo_amount, cargo_type);
	printf("amount to load: %d, amount taken: %d\n", -amount, -cargo_amount);
	if( (cargo_amount - loaded) > 0 )
		src->loadCargo(cargo_amount - loaded,cargo_type);
	return cargo_amount;
}

void FoundationCore::addFleet(Fleet *f)
{
	/* 	first, check if there is already a fleet with the same id
		it might for a network update */
	Fleet *old_fleet;
	old_fleet = getFleet( f->getId() );
	if( old_fleet != NULL)
	{
		*old_fleet = *f;
		return;
	}

	/* if not, we'll just add it to the end */
	fleet.push_back(f);
	printf("new fleet size: %d\n", fleet.size() );
}

int FoundationCore::setDest(int f, Point p)
{
	getFleet(f)->setDest(p);
}

/** Returns visibility level of the pos from
planet planet_id */
int FoundationCore::canSee(int planet_id, Point pos)
{
	pos = pos - planet[planet_id]->pos;
/* 	for now, we'll hard code a distance.  Later, we'll do a check
	based on speed, distance, fleet's cloaking abilities and the
	planet's owner's technology and return different levels
	ranging from only position to full info on the fleet
	(ships in fleet, cargo, other waypoints)
*/
	if (pos.length() < VIS_LEN)
	{
		printf("I can see you\n");
		return 1;
	}
	printf("I can't see you\n");
	return 0;
}

int FoundationCore::sendPlanet(struct sockaddr_in addr, SolarSystem *s,
					int info_level)
{
		int len;
		struct update_part st_planet_part;
		st_planet_part.planet = s->getId();
		st_planet_part.current_item = planet[s->getId()]->getFrontId();
		st_planet_part.progress = planet[s->getId()]->getProgress();	
		memcpy(st_planet_part.cargo, planet[ s->getId() ]->cargo,
				sizeof(int)*CARGO_NUM);
	
		len += sizeof(st_planet_part);

		memcpy(st_planet_part.building, planet[ s->getId() ]->building,
			sizeof(bool)*BUILDING_NUM);
		
		ne.sendpacket(addr, PLANET_UPDATE,
			PLANET_UPDATE_PART, (char *)&st_planet_part, len);
}

int FoundationCore::sendFleet(struct sockaddr_in addr, Fleet *f,
					int info_level)
{
/* 	we'll eventually switch this to a case statement and prune the data
	based on the info level.  The info level will only change once
	technology has been added to the game.
	For now, we just send full info */
	int len;
	char *buf;
	Fleet f_info;

	buf = (char *)malloc(MAX_PACKET_SIZE);
/*
	f_info = *f;
	f_info.clearDest();
	f_info.pos = f->pos;
	f_info.setDest( f_info.pos );
	f_info.setDest( f->getDest(1) );
*/
	len = f->getRaw(buf);
	ne.sendpacket(addr, FLEET_UPDATE, FLEET_ADD, buf, len);
	printf("done sendFleet %d\n", len);
	free(buf);
}
