/***************************************************************************
                          foundationcore.h  -  description
                             -------------------
    begin                : Sun May 21 2000
    copyright            : (C) 2000 by Blake Friesen
    email                : umfrie64@cc.umanitoba.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONFLICTCORE_H
#define CONFLICTCORE_H

#include <ship.h>
#include <shipInfo.h>
#include <system.h>
#include <vector>
#include <unit_def.h>
#include <fleet.h>
#include <sys/socket.h>
#include <netinet/ip.h>
#include <netdb.h>

#include <foundationcore.h>
#include <netengine.h>
#include <player.h>

#include "network/network.h"
/**base class of 'Foundation' (user game) and FoundationServer,
the network server.  This is made up of common
data structures and functions.
  *@author Blake Friesen
  */

#define VIS_LEN 1000

class FoundationCore {
public: 
	FoundationCore();
	virtual ~FoundationCore();
	void setBuildItem(int planet, int group, int type, int quantity);

  /** a major loop
waits for network packet and handles
accordingly
 */
  void net_wait();
  /** finds a planet by it's coordinates
does a binary search and assumes
that the planets are sorted
not yet implemented
 */
	int find_planet(Point p);
protected:
/* the member variables are protected which means that the "Foundation"
	and "FoundationServer" classes can access them (good) but so can
	the FoundationGUI class(es) (bad). */
	int started;
	vector<SolarSystem *> planet;
	NetEngine ne;
	Player player[MAX_CLIENTS];
/* network reception handling routines */
	void loadnames();
	virtual void handle_chat_msg(char *buf, int len);
	virtual void handle_game_join(sockaddr_in addr, char *buf, int len);
	virtual void handle_game_start(sockaddr_in addr, char *buf, int len);

	virtual void handle_planet_info(char *buf, int len);
  /** handles incoming updates about planet
changes.  Should only be used by the
client. */

	virtual void handle_planet_part(sockaddr_in addr, char *buf, int len);
	virtual void handle_prod_request(sockaddr_in addr, char *buf, int len);

/** handles new fleet addition, meant to be
	used by the client */
	virtual void handle_fleet_add(sockaddr_in addr, char *buf);
	virtual int add_waypoint(Fleet *f, int p);
	virtual void handle_add_waypoint(sockaddr_in addr, char *buf, int len);
	virtual void handle_transfer_cargo(sockaddr_in addr, char *buf, int len);
/* added from ship info */
public:
	int 			addShip(int s, Point p, int owner);
	int 			setDest(int f, Point dest);
	int 			xfer(Platform *a, Platform *b, int amount, int cargo);	
	int 			loadFleet(int f, int n, int t);
	Fleet *			getFleet(int f);
	vector<Platform *> getPlatform(Point p);
	vector<Fleet *> getFleet(Point p);
	vector<Fleet *> getFleetOwner(int owner);
	void addFleet(Fleet *f);

protected:
	void loadShipSpecs();

  /** Returns visibility level of the pos from
planet planet_id */
	int canSee(int planet_id, Point pos);

	int sendFleet(struct sockaddr_in addr, Fleet *f, int info_level);
	int sendPlanet(struct sockaddr_in addr, SolarSystem *s, int info_level);
	vector<Ship> shiptype_list;
	vector<Fleet *> fleet;
};

#endif
