/***************************************************************************
                          StarMap.cpp  -  description
                             -------------------
    begin                : Tue Jan 25 2000
    copyright            : (C) 2000 by Blake Friesen
    email                : umfrie64@cc.umanitoba.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "StarMap.h"
#include <unistd.h>

#include <point.h>
#include <stdlib.h>
#include <iostream.h>
#include <math.h>
#include <stdio.h>

#include <GL/glu.h>
#include <GL/gl.h>

StarMap::StarMap(int a, int b, int c, int d) : puObject(a, b, c, d)
{
	rotX = rotY = 0;
	Xdelta = Ydelta = 0;
	planet = NULL;
	draw_plane = 0;
}

void StarMap::init(){
	glEnable(GL_FOG);
	glEnable(GL_DEPTH_TEST);
	glFogi(GL_FOG_MODE, GL_LINEAR);
	glFogf(GL_FOG_DENSITY, 0.3);
	glFogi(GL_FOG_START, 600);
	glFogi(GL_FOG_END, 2500);
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glFrustum(-400, 400, -400, 400, 700, -1750);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();	
	glLoadIdentity();
	makeOwner();
	makeStar();
	makeSelect();
	makeFleet();
	glTranslatef(0,0,-1400);
	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
}

void StarMap::setupView()
{
	glFrustum(-400.0, 400.0, -400.0, 400.0, 700.0, 1750.0);
	glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
	glTranslatef(0.0,0.0,-1400.0);
	glPushMatrix();
	glRotatef( (int)rotY/8, 1.0, 0.0, 0.0);
	glRotatef( (int)rotX/8, 0.0, 1.0, 0.0);
	glGetFloatv(GL_MODELVIEW_MATRIX, coef);
	glPopMatrix();
}

void StarMap::findPlanet(int mx, int my)
{
	int planet_selected;
	planet_selected = findPlatform( (vector<Platform *> *)planet, mx, my);
	if(planet_selected == -1) return;
	planet_clicked( planet_selected );
}

void StarMap::findFleet(int mx, int my)
{
    int fleet_selected;
	fleet_selected = findPlatform( (vector<Platform *> *)fleet, mx, my);
	if(fleet_selected == -1) return;
	fleet_clicked( (*fleet)[fleet_selected]->getId() );
}



int StarMap::findPlatform(vector<Platform *> *platform, int mx, int my)
{
/* returns the index in the vector of the item clicked */
	down[0] = 1;
	GLuint selectBuf[PLANET_NUM*4];
	int i,hits=0;
      /* set up the selection buffer (4 ints per hit) */
      glSelectBuffer(PLANET_NUM*4, selectBuf);
	/* change the render mode to selection and clear the name buffer */
      glRenderMode(GL_SELECT);
      glInitNames();
      glPushName(0);
	/* must have something on stack before callins glLoadName */

      glMatrixMode(GL_PROJECTION); glPushMatrix();
      glMatrixMode(GL_MODELVIEW); glPushMatrix();

   GLint viewport[4];
   glViewport(0, abox.min[1], abox.max[0], abox.max[1]); // new viewport for pui
   glGetIntegerv(GL_VIEWPORT,viewport);
   // Create the projection matrix

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
    gluPickMatrix( mx, my, 10, 10, viewport);
      // so that we don't lose our projection/model matrix, push both
	setupView();

//		printf("size of platform %d\n", platform->size() );
		for(int x = 0;x < platform->size() ;x++)
		{
		   	int colour = x % 6;
			glLoadName(x);
			draw_point( (*platform)[x]->getPos(), starPic[colour]);
	   	}

      // switch to render mode, the return value is the number of hits.
    hits=glRenderMode(GL_RENDER);
   // process the selection buffer
	cout << "number of hits: " << hits << endl;
	if (hits>0){
		for (i=0;i<hits;i++) cout << "hit: " << selectBuf[(i*4)+3] << endl;
	    return(selectBuf[3]);
     }

      // pop the projection/model matrix
    glMatrixMode(GL_PROJECTION); glPopMatrix();
    glMatrixMode(GL_MODELVIEW); glPopMatrix();
	glViewport(0,0,gl_width, gl_height);
	return -1;
}


void StarMap::setFleet(vector<Fleet *> *f)
{
	fleet = f;
}


void StarMap::paintGL()
{
/* 	this function is somewhat (very)ugly.... needs to be optimized and
	also broken up */
//   cout << "\npainting\n";
    /*
	glEnable(GL_ALPHA);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);      // alpha blending
	glEnable(GL_ALPHA_TEST);
  */
	glEnable(GL_FOG);
	glEnable(GL_DEPTH_TEST);
    glFogi(GL_FOG_MODE, GL_LINEAR);
    glFogf(GL_FOG_DENSITY, 0.3);
    glFogi(GL_FOG_START, 600);
    glFogi(GL_FOG_END, 2500);
	rotX += Xdelta;
	rotY += Ydelta;
	glMatrixMode(GL_PROJECTION); glPushMatrix();
	glMatrixMode(GL_MODELVIEW); glPushMatrix();
	glEnable(GL_DEPTH_TEST);
	glViewport(0, abox.min[1], abox.max[0], abox.max[1]);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glFrustum(-400.0, 400.0, -400.0, 400.0, 700.0, 1750.0);
	glMatrixMode(GL_MODELVIEW);

	glLoadIdentity();
	glTranslatef(0.0,0.0,-1400.0);
      glPushMatrix();
        glRotatef( (int)rotY/8, 1.0, 0.0, 0.0);
        glRotatef( (int)rotX/8, 0.0, 1.0, 0.0);
	glColor3f(0.1, 0.1, 1.0);

/* I should turn this into a display list */
	if(draw_plane)
	{
       	glBegin(GL_LINE_LOOP);
		glVertex3i(-375,0,-375);
		glVertex3i(-375,0,375);
		glVertex3i(375,0,375);
		glVertex3i(375,0,-375);
		glEnd();
		glBegin(GL_LINES);
    	for(int x = 0;x<PLANET_NUM;x++)
		{
			Point p = (*planet)[x]->getPos();
			glVertex3i(0, 0, 0);
			glVertex3i(p.x, 0, p.z);
			glVertex3i(p.x, 0, p.z);
			glVertex3i(p.x, p.y, p.z);
		}
		glEnd();
	}
	else glutWireCube(750.0);
	
        glGetFloatv(GL_MODELVIEW_MATRIX, coef);
      glPopMatrix();

	glColor3f(0.2,0.2,1.0);

	for(int x = 0;x<PLANET_NUM;x++){
	   	int color = x % 6;
	draw_point( (*planet)[x]->getPos(), starPic[ color ]);
	if(x == current_planet) draw_point( (*planet)[x]->getPos(), selected);
			draw_point( (*planet)[x]->getPos(), ownerPic[ (*planet)[x]->owner ]);
   	}

	for(int x = 0;x < fleet->size();x++)
	{
    	draw_point( (*fleet)[x]->getPos(), fleet_pic);
		vector<Point> dest = (*fleet)[x]->getDest();
	/* tmp hack */
      glPushMatrix();
        glRotatef( (int)rotY/8, 1.0, 0.0, 0.0);
        glRotatef( (int)rotX/8, 0.0, 1.0, 0.0);

		glBegin(GL_LINE_STRIP);
		for(int i = 0;i<(int)dest.size();i++)
		{
			glVertex3i(dest[i].x, dest[i].y, dest[i].z);
		}
		glEnd();
      glPopMatrix();
	}

	glDisable(GL_FOG);
    glMatrixMode(GL_PROJECTION); glPopMatrix();
    glMatrixMode(GL_MODELVIEW); glPopMatrix();
	glViewport(0,0,gl_width,gl_height);
}

Point StarMap::draw_point(Point p, int pic){
	Point pos;
    glPushMatrix();
	float x = (float)p.x;
    float y = (float)p.y;
  	float z = (float)p.z;
    int a = (int)(x * coef[0] + z * coef[8] );
    int b = (int)(x * coef[1] + y * coef[5] + z * coef[9]);
	int c = (int)(x * coef[2] + y * coef[6] + z * coef[10]);
    glTranslatef(a, b, c);
    glCallList(pic);
	glPopMatrix();
	return pos;
}

void StarMap::makeOwner(){
	
	for(int i=0;i<16;i++)
	{
		i++;
		int list = glGenLists( 1 );
		glNewList(list, GL_COMPILE);
		glColor3ub( (i*177) % 255,(i*47) % 255, (i*1937) % 255);
//		printf("setting colour: %d %d %d\n",
//			(i*177) % 255,(i*47) % 255, (i*1937) % 255);
		glBegin(GL_TRIANGLES);
		glVertex3f(0,12,0.1);
		glVertex3f(12,12,0.1);
		glVertex3f(12,0,0.1);
		glEnd();
		glEndList();
		i--;
		ownerPic[i] = list;
	}
}

void StarMap::makeSelect(){
	// 24 x 24, indicates which start has been selected
	int list = glGenLists( 1 );
	glNewList(list, GL_COMPILE);
	glColor3ub(0,255,0);
	glTranslatef(-12,-12,0);
	glBegin(GL_TRIANGLES);
		glVertex3f(8,0,0.1);
		glVertex3f(0,0,0.1);
		glVertex3f(0,8,0.1);
	glEnd();
        glBegin(GL_TRIANGLES);
                glVertex3f(16,0,0.1);
                glVertex3f(24,0,0.1);
                glVertex3f(24,8,0.1);  
        glEnd();
        glBegin(GL_TRIANGLES);
                glVertex3f(24,16,0.1);
                glVertex3f(24,24,0.1);
                glVertex3f(16,24,0.1);  
        glEnd();
        glBegin(GL_TRIANGLES);
                glVertex3f(0,16,0.1);
                glVertex3f(0,24,0.1);
                glVertex3f(8,24,0.1);  
        glEnd();

	glEndList();
	selected = list;
}

void StarMap::makeFleet(){
	int list = glGenLists( 1 );
	glNewList(list, GL_COMPILE);
	glColor3ub(0,255,255);
        glBegin(GL_TRIANGLES);
                glVertex3f(28,0,0); 
                glVertex3f(0,0,0);  
                glVertex3f(0,-28,0); 
        glEnd();
        glEndList();
        fleet_pic = list;
}


int StarMap::makeStar(){ 
      
char starcolors[6][3] =
	{
		{ 178, 255, 183 },
		{ 255, 255, 192 },
		{ 255, 210, 210 },
		{ 187, 199, 255 },
		{ 240, 240, 255 },
		{ 255, 255, 255 }
	};
	      	for(int x=0;x<6;x++){
                int list = glGenLists( 1 );
                glNewList(list, GL_COMPILE);
                glBegin( GL_TRIANGLE_FAN);
		      	glColor3ub(starcolors[x][1],starcolors[x][2], starcolors[x][3]);
                glVertex3f(0, 0, 0);
	            glColor4ub(15, 15, 15, 0);
            	for(int i = 0; i<= 360;i += 45){
                	glVertex3f( 12*cos(i/57.29578), 12*sin(i/57.29578), 0.0);
            		}
                glEnd();
                glEndList();
                starPic[x]=list;
        }                                                     	
	return 0;
}

void StarMap::setPlanet( vector<SolarSystem *> *p){
//	cout << "Planet set...\n";
//	printf("p: %d\n", p);
	planet = p;
}

void StarMap::setStar(int s){
		current_planet = s;
}

void StarMap::updateGL(){
	paintGL();
}

int StarMap::getCurrentPlanet()
{
	return current_planet;
}


void StarMap::draw(int a, int b)
{
	paintGL();
}

int StarMap::checkKey(int key, int updown )
{
	switch(key)
	{
	case(49): Xdelta--;
		  Ydelta--;
		break;
	 case(50): Ydelta--;
                break;
	 case(51): Xdelta++;
                  Ydelta--;
                break;
	 case(52): Xdelta--;
                break;
	 case(53): Xdelta = 0;
                  Ydelta = 0;
                break;
	 case(54): Xdelta++;
                break;
	 case(55): Xdelta--;
                  Ydelta++;
                break;
	 case(56):
                  Ydelta++;
                break;
	 case(57): Xdelta++;
                  Ydelta++;
                break;
	 case('p'):
	 case('P'):
			draw_plane = !draw_plane;
			break;
	}
	return 0;
}

int StarMap::checkHit( int button, int updown, int x, int y)
{
	printf("checkHit called: button: %d, status: %d, x: %d, y: %d\n", button, updown, x, y);
	//printf("action box: %d %d %d %d\n", abox.min[0], abox.min[1], abox.max[0], abox.max[1]);
	if((x > abox.min[0]) && (x < abox.max[0]) && (y > abox.min[1]) && (y < abox.max[1]) ) {
	if(updown == 0)
	{
		mx = x;
		my = y;
		if(button == 0) findPlanet(x, y); /* left mouse button */
		if(button == 2) findFleet(x, y);  /* right mouse button */
	}
	if(updown == 255)
	{
      int dx = mx - x;
      int dy = my - y;
      mx = x;
      my = y;
      rotX -= dx;
      rotY -= dy;
      paintGL();
	}
	}
	else return 0;
	return 1;
}
