/***************************************************************************
                          cargotransfer.cpp  -  description
                             -------------------
    begin                : Thu Jul 27 2000
    copyright            : (C) 2000 by Blake Friesen
    email                : umfrie64@cc.umanitoba.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cargotransfer.h"

CargoTransfer::CargoTransfer(int a, int b, int c, int d) : UserView(a, b, c, d)
{
	vector<int> full_list;
	vector<GLuint> tex_list;

	dest = source = NULL;
	cargo_choice = -1;

	puGroup *pug = new puGroup(a,b);

	slider = new puSlider( 2*size_mult,2*size_mult,
		188*size_mult, 14*size_mult, 0);
	slider->setCallback( ( void (*)(puObject *) )slider_cb );
	slider->setUserData( (void *)this );

	platform_name = new puText(0,60*size_mult);
	platform_name->setLabel("PlatformID");
	platform_name->setColour(PUCOL_LABEL, 1.0, 1.0, 1.0 );

	cargoChoice = new ProdChoice(0, 15*size_mult, 193*size_mult,
		68*size_mult);
	cargoChoice->setStyle( PUSTYLE_NONE );

	cargoChoice->width=2;
	load_images("cargo_pics.txt", &tex_list);
	cargoChoice->setTexList(tex_list);

	for(int i=0;i<CARGO_NUM;i++) full_list.push_back(i);
	cargoChoice->setDisplayList(full_list);

	for(int i=0;i<CARGO_NUM;i++)
	{
		float x_start, y_start;
		getCargoLoc(&x_start, &y_start, i);
		cargo[i] = new puText(x_start, y_start - 148*size_mult); /* tmp hack */
		cargo[i]->setColour(PUCOL_LABEL, 1.0, 1.0, 1.0 );
		cargo[i]->setLabel("0");
	}

	pug->close();

	memset(cargo_offset, 0, sizeof(int) * CARGO_NUM);
    cargoChoice->image_clicked.connect(slot(this, &CargoTransfer::cargoClicked));
}

CargoTransfer::~CargoTransfer()
{
	delete slider;
}

void CargoTransfer::clearOffset()
{
	memset(cargo_offset, 0, sizeof(int) * CARGO_NUM);
}

void CargoTransfer::draw(int dx, int dy)
{
   if( !visible ) return;
	makeCargoLabels( dest, cargo_offset );
   	if (dest!=NULL)
	{
	   	char *label = (char *)malloc(32);
		sprintf(label, "Platform: %d", dest->getId() );
		platform_name->setLabel(label);
    }

	glEnable(GL_DEPTH_TEST);
	abox.draw( dx, dy, PUSTYLE_SMALL_BEVELLED, colour, isReturnDefault() );
//	glTranslatef(0.0,0.0,0.1);
	glDisable(GL_DEPTH_TEST);
}

void CargoTransfer::getCargoOffset(int *neg_offset)
{
	int i;
	for(i = 0;i<CARGO_NUM;i++)
	{
		neg_offset[i] = - cargo_offset[i];
	}
}

void CargoTransfer::hide()
{
	visible = 0;
	slider->hide();
	platform_name->hide();
	cargoChoice->hide();
	int i;
	for(i = 0;i < CARGO_NUM;i++)
	{
		cargo[i]->hide();
	}

}

void CargoTransfer::reveal()
{
	visible = 1;
	slider->reveal();
	platform_name->reveal();
	cargoChoice->reveal();

	int i;
	for(i = 0;i < CARGO_NUM;i++)
	{
		cargo[i]->reveal();
	}

}

void CargoTransfer::setDest(Platform *newdest)
{
	dest = newdest;
}

void CargoTransfer::setSource(Platform *newsource)
{
	source = newsource;
}


void CargoTransfer::cargoClicked(int cargo_clicked)
{
	int max_cargo;
	int source_cargo, dest_cargo;
	cargo_choice = cargo_clicked;
	if (source == NULL | dest == NULL) return;
	if (cargo_clicked == cargoChoice->get_selected() ) return;
/*	
	if(cargoChoice->get_selected() != -1)
	{
		cargo_offset[cargo_clicked] = - slider->getValue() *
			(dest->cargo[cargo_clicked] + source->cargo[cargo_clicked]);
		printf("cargo offset: %d\n", cargo_offset[cargo_clicked]);
	}
*/
/* need to set up the slider position */
	source_cargo = source->cargo[cargo_clicked] - cargo_offset[cargo_clicked];
	dest_cargo = dest->cargo[cargo_clicked] + cargo_offset[cargo_clicked];
	if(dest->isFleet() )
	{
		max_cargo = source_cargo + dest_cargo;
		if( max_cargo == 0)
				slider->setValue( 0 );
		else
		{
			max_cargo += cargo_offset[cargo_choice];
			slider->setValue( dest_cargo / max_cargo );
		}
	}
	else
	{
		max_cargo = source_cargo;
		if ( dest_cargo < source->cargoFree() )
		{
			max_cargo += dest_cargo;
		}
		else
		{
			max_cargo += source->cargoFree();
		}
		if( max_cargo == 0)
				slider->setValue( 0 );
		else
		{
			float slider_val;	
			slider_val = (float)source_cargo / (float)max_cargo;
//			printf("source_cargo: %d, max_cargo: %d\n", source_cargo, max_cargo);
//			printf("slider val: %f\n", slider_val);
			slider->setValue( slider_val );
		}
	}
	cargoChoice->set_selected( cargo_clicked );
}


void CargoTransfer::slider_cb(puObject *cb)
{	
	float val;

	cb->getValue( & val );
	CargoTransfer *ct = (CargoTransfer *) cb->getUserData(); /* ugly hack */
	ct->adjustCargo( val );
}

void CargoTransfer::adjustCargo( float val )
{
	if( cargo_choice != -1)
	{
		cargo_offset[cargo_choice] = source->cargo[cargo_choice] - (int)(val *
			(dest->cargo[cargo_choice] + source->cargo[cargo_choice]) );
		printf("cargo offset: %d\n", cargo_offset[cargo_choice]);
	}
}
