/***************************************************************************
                          fleetview.cpp  -  description
                             -------------------
    begin                : Tue May 2 2000
    copyright            : (C) 2000 by Blake Friesen
    email                : umfrie64@cc.umanitoba.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "fleetview.h"

FleetView::FleetView(int a, int b, int c, int d) : UserView(a, b, c, d)
{
	unsigned int i;
	fleet = NULL;
	puGroup *pug = new puGroup(a,b);
	vector<int> full_list;
	vector<GLuint> tex_list;
	cargoChoice = new ProdChoice(0, 160*size_mult, 193*size_mult,
		217*size_mult);
	commandChoice = new ProdChoice(0, 142*size_mult, 193*size_mult,
		160*size_mult);
	cargoTrans = new CargoTransfer(0, 62*size_mult, 193*size_mult,
		142*size_mult);
	shipChoice = new ProdChoice(0, 0, 193*size_mult, 62*size_mult);

	cargoChoice->width=2;
	shipChoice->width=12;
	commandChoice->width=5;

	load_images("command_pics.txt", &tex_list);
	commandChoice->setTexList(tex_list);
	full_list.clear();
	for(i=0;i<tex_list.size();i++)
		 full_list.push_back(i);
	commandChoice->setDisplayList(full_list);
	tex_list.clear();

	load_images("cargo_pics.txt", &tex_list);
	cargoChoice->setTexList(tex_list);
	tex_list.clear();

	load_images("ship_pics.txt", &tex_list);
	shipChoice->setTexList(tex_list);

	for(int i=0;i<CARGO_NUM;i++) full_list.push_back(i);
	cargoChoice->setDisplayList(full_list);
	full_list.clear();

	fleet_name = new puText(10*size_mult,218*size_mult);
	fleet_name->setLabel("FleetID");
	fleet_name->setColour(PUCOL_LABEL, 1.0, 1.0, 1.0 );

	fleet_loc = new puText(66*size_mult,218*size_mult);
	fleet_loc->setLabel("(xxx, yyy, zzz)");
	fleet_loc->setColour(PUCOL_LABEL, 1.0, 1.0, 1.0 );
	for(int i=0;i<CARGO_NUM;i++)
	{
		float x_start, y_start;
		getCargoLoc(&x_start, &y_start, i);
		cargo[i] = new puText(x_start, y_start);
		cargo[i]->setColour(PUCOL_LABEL, 1.0, 1.0, 1.0 );
		cargo[i]->setLabel("0");
	}
	pug->close();

    commandChoice->image_clicked.connect(slot(this, &FleetView::commandClicked));
	dest = NULL;
	cargoTrans->hide();
}

/*
void FleetView::getCargoLoc(float *x, float *y, int i)
{
	*x = (18+(93* (i % 2)))*size_mult;
	*y = (356-(160+(6*(i-(i%2)))))*size_mult;
}
 */

Fleet * FleetView::get_fleet()
{
	return fleet;
}

void FleetView::set_fleet(Fleet *f)
{
	fleet = f;
//	printf("setting fleet: %d\n", fleet->getId() );
	update_shiplist();
	commandChoice->set_selected( -1 );
	cargoTrans->setSource( (Platform *) fleet );
}

void FleetView::makeFleetLabels()
{
	if(fleet == NULL) return;
	Point fleet_pos;
	char *label = (char *)malloc(32);
	sprintf(label, "Fleet: %d", fleet->getId() );
	fleet_name->setLabel(label);

	label = (char *)malloc(32);
	fleet_pos = fleet->getPos();
	sprintf(label, "(%d, %d, %d)", fleet_pos.x,fleet_pos.y, fleet_pos.z );
	fleet_loc->setLabel(label);
}

void FleetView::draw(int dx, int dy)
{
	int cargo_offset[CARGO_NUM];
	cargoTrans->getCargoOffset( cargo_offset );
	makeCargoLabels(fleet, cargo_offset);
	makeFleetLabels();

	glEnable(GL_DEPTH_TEST);
	glTranslatef(0.0,0.0,-0.1);
	abox.draw ( dx, dy, PUSTYLE_SMALL_BEVELLED, colour, isReturnDefault() ) ;
	glTranslatef(0.0,0.0,0.1);
	glDisable(GL_DEPTH_TEST);
}

void FleetView::setNextCargoDest()
{
/* bottom info */	
	int i;
	i = 0;
	if(dest == NULL)
		dest = fleet;
	vector<Platform *> platforms = getPlatforms( fleet->pos );

	if (platforms.size() == 1)
	{
		dest = NULL;
		return;
	}

	for(i = 0;i < (int)platforms.size(); i++)
	{
		if (platforms[i]->getId() == dest->getId()) break;
	}	
	i++;
	if(i >= (int)platforms.size())
		i = 0;
//	printf("i: %d\n", i);
//	fflush( stdout );
	if ( platforms[i]->getId() == fleet->getId() )
		i++;

	if(i >= (int)platforms.size())
		i = 0;
	dest = platforms[i];
	cargoTrans->setDest( dest );
	cargoTrans->setSource( (Platform *) fleet );
}

void FleetView::setNextCargoSource()
{
/* top info */
	int i;
	i = 0;
	vector<Platform *> platforms = getPlatforms( fleet->pos );
//	printf("%d items here\n", platforms.size() );
	fflush( stdout );
	for(i = 0;i < (int)platforms.size(); i++)
	{
		if (platforms[i]->getId() == fleet->getId()) break;
	}	
	i++;
	if(i >= (int)platforms.size()) i = 0;
/* if there is a planet in this list, it is always the last one in the list */
	if ( !platforms[i]->isFleet() ) i = 0;

/* shouldn't ever happen */
	if ( !platforms[0]->isFleet() )
		fleet = NULL;
	else
		fleet = (Fleet *) platforms[i];
}

void FleetView::commandClicked(int command)
{
/*
	if(command != CARGO_TRANSFER)
	{
			cargoTrans->hide();
	}
*/
	switch (command)
	{
		case CARGO_TRANSFER:
//			printf("cargo transfer clicked\n");
			if(cargoTrans->isVisible())
			{
				cargoTrans->hide();
				commandChoice->set_selected( -1 );
				if(fleet != NULL && dest != NULL)
				{
					int cargo_offset[CARGO_NUM];
					cargoTrans->getCargoOffset( cargo_offset );
					cargoRequest( dest, fleet, cargo_offset);
					cargoTrans->clearOffset();
				}
			}
			else
			{
				cargoTrans->reveal();
				commandChoice->set_selected( CARGO_TRANSFER );
				cargoTrans->clearOffset();
			}
			break;
		case NEXT_FLEET:
			if (commandChoice->get_selected() == CARGO_TRANSFER)
				setNextCargoDest();
			else setNextCargoSource();
			break;
		case PREV_FLEET:
			break;

		default:
//			printf("unknown command %d\n", command);
			commandChoice->set_selected(command);
			vector<Platform *> platforms_here = getPlatforms( fleet->pos );
//			printf("%d objects here\n", platforms_here.size() );
	}
}

int FleetView::get_command_state()
{
	return commandChoice->get_selected();
}

int FleetView::checkHit( int button, int updown, int x, int y)
{
//	printf("fleetview checkHit called: button: %d, status: %d, x: %d, y:
//	%d\n", button, updown, x, y);
	//printf("action box: %d %d %d %d\n", abox.min[0], abox.min[1],
	//	abox.max[0], abox.max[1]);
//	if((x > abox.min[0]) && (x < abox.max[0]) && (y > abox.min[1]) &&
//		(y < abox.max[1]) ) printf("inside fleetview\n");
//	else return 0;
	return 0;
}

/** Updates the ships shown
 */
void FleetView::update_shiplist()
{
	int i = 0;
	vector<int> display_list;
	Ship *ship;

	while( (ship = fleet->getShip( i ) ) != NULL)
	{
		printf("adding ship to display list\n");
		display_list.push_back( ship->getType() );
		i++;
	}
	printf("size of ship display list: %d\n", display_list.size() );
	shipChoice->setDisplayList( display_list );
}
