#include <stdio.h>
#include <foundationgl.h>

FoundationGL::FoundationGL( int argc, char **argv){
	int smheight = (int)(gl_height*0.8);
	int smwidth = (int)(gl_width*0.7);
	starmap = new StarMap(0, gl_height-smheight, smwidth, smheight);
//	printf("starmap: %d, %d, %d, %d\n", 0, gl_height-smheight, smwidth, gl_height);
	starmap->setPlanet( &planet );
	starmap->setFleet( &fleet  );
	starmap->init();
//	printf("planet: %d\n", planet);
	planetview = new PlanetView(smwidth, (int)(gl_height/2), gl_width, gl_height);
	planetview->setbuilditem.connect(slot(this, &FoundationGL::setbuilditem));
    starmap->planet_clicked.connect(slot(this, &FoundationGL::planet_clicked));
    starmap->fleet_clicked.connect(slot(this, &FoundationGL::fleet_clicked));

	fleetview = new FleetView(smwidth, 0, gl_width, (int)(gl_height/2));
	fleetview->getPlatforms.connect(slot(this, (getPlatformByPos )&FoundationCore::getPlatform) );
	fleetview->cargoRequest.connect(slot(this, (cargoRequestType )&Foundation::sendTransferRequest) );
	commandline = new CommandLine(0, 0,smwidth,gl_height-smheight);
    commandline->command_entered.connect(slot(this, &FoundationGL::command_entered));
	frames = 0;
	last_update = 0;
	planet_clicked(0);
}

FoundationGL::~FoundationGL()
{
}

void FoundationGL::display(){
	if( time(NULL) > last_update)
	{
		last_update = time(NULL);
		printf("fps: %d\t%d\n", last_update, frames);
		frames = 0;
	}
	frames++;
	glClearColor ( 0.0, 0.1, 0.0, 1.0 ) ;
	glClear      ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ) ;
	puDisplay () ;
	glutSwapBuffers   () ;
	glutPostRedisplay () ;
}

void FoundationGL::planet_clicked(int p)
{
/*  this action is context sensitive, depending on what the fleetview
	selection is.
*/
	printf("Planet %d clicked.\n", p);
	switch( fleetview->get_command_state() )
	{
	case ADD_PATH:
		/* leave ADD_PATH until later */
	case ADD_WAYPOINT:
			add_waypoint( fleetview->get_fleet(), p );
			break;
	case TRANS_CARGO:
			break;

	default:
		planetview->setPlanet(planet[p]);
		starmap->setStar(p);
		break;

	}
}

void FoundationGL::fleet_clicked(int p)
{
	printf("FleetID %d clicked.\n", p);
	fleetview->set_fleet( FoundationCore::getFleet(p) );
}


void FoundationGL::command_entered(char *command)
{
	printf("command entered: %s\n", command);
	send_command(command);
}

void FoundationGL::setbuilditem(int group, int type, int quantity)
{
	setBuildItem(starmap->getCurrentPlanet(), group, type, quantity);
}

void FoundationGL::tell_player(char *msg)
{
	commandline->tell_player(msg);
}

/** extended from Foundation class
	shows player a specific planet even
	if they don't select it, meant for network
	or other game events  */
void FoundationGL::show_planet(int planet_id)
{
	planet_clicked(planet_id);
}

void FoundationGL::doUIrequest()
{
	requestPlanetCargoUpdate( starmap->getCurrentPlanet() );
}
