/***************************************************************************
                          planetview.cpp  -  description
                             -------------------
    begin                : Sun Apr 23 2000
    copyright            : (C) 2000 by Blake Friesen
    email                : umfrie64@cc.umanitoba.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <vector>

#include "planetview.h"

PlanetView::PlanetView(int a, int b, int c, int d) : UserView(a, b, c, d)
{
	printf("PlanetView initializing...\n");
	puGroup *pug = new puGroup(a,b);

	planet_name = new puText(10*size_mult,220*size_mult);
	planet_name->setLabel("Planet Name");
	planet_name->setColour(PUCOL_LABEL, 1.0, 1.0, 1.0 );
/* setup the multi-image selection boxes, aka ProdChoice objects */
	vector<int> full_list;
	vector<GLuint> tex_list;
	for(int i=0;i<CARGO_NUM;i++) full_list.push_back(i);
/*	old
	buildChoice = new ProdChoice(5*size_mult,85*size_mult, 62*size_mult,
		145*size_mult);
	shipChoice  = new ProdChoice(64*size_mult,85*size_mult, 121*size_mult,
		145*size_mult);
	buildQueue  = new ProdChoice(123*size_mult,85*size_mult, 192*size_mult,
		145*size_mult);
	cargoChoice = new ProdChoice(5*size_mult, 160*size_mult, 192*size_mult,
		217*size_mult);
*/
	buildChoice = new ProdChoice(0,85*size_mult, 60*size_mult,
		145*size_mult);
	shipChoice  = new ProdChoice(60*size_mult,85*size_mult, 120*size_mult,
		145*size_mult);
	buildQueue  = new ProdChoice(120*size_mult,85*size_mult, 193*size_mult,
		145*size_mult);
	cargoChoice = new ProdChoice(0, 160*size_mult, 193*size_mult,
		217*size_mult);
	buildChoice->width=4;
	shipChoice->width=4;
	buildQueue->width=6;
	cargoChoice->width=2;
/* telling the production choice boxes what they should display */
	load_images("ship_pics.txt", &tex_list);
	shipChoice->setTexList(tex_list);
	tex_list.clear();

	load_images("cargo_pics.txt", &tex_list);
	cargoChoice->setTexList(tex_list);
	tex_list.clear();

	load_images("build_pics.txt", &tex_list);
	buildChoice->setTexList(tex_list);

/* the buildqueue display widget can display images of any production
	type so we must set it up to be able to do so */
/* unfortunately, order here is very important... :/  */
	tex_list.clear();
	load_images("cargo_pics.txt", &tex_list);
	load_images("build_pics.txt", &tex_list);
	load_images("ship_pics.txt", &tex_list);
	buildQueue->setTexList(tex_list);

	cargoChoice->setDisplayList(full_list);
    printf("\tplanet_name init...\n");
	printf("\tcargo_img_name init...\n");
	for(int i=0;i<CARGO_NUM;i++)
	{
		float x_start, y_start;
		cargoChoice->getPosition(&x_start, &y_start, i);
		cargo[i] = new puText(x_start+15.0*size_mult, y_start+154.0*size_mult);
		cargo[i]->setColour(PUCOL_LABEL, 1.0, 1.0, 1.0 );
		cargo[i]->setLabel("0");
	}
	pug->close();

	/* setup production choice boxes */
    cargoChoice->image_clicked.connect(slot(this, &PlanetView::cargoClicked));
    buildChoice->image_clicked.connect(slot(this, &PlanetView::buildClicked));
    shipChoice->image_clicked.connect(slot(this, &PlanetView::shipClicked));
	printf("PlanetView done initializing...\n");
}


void PlanetView::draw(int a, int b)
{
 //	printf("planet viewport: %d, %d, %d, %d\n", abox.min[0],
//		abox.min[1], abox.max[0], abox.max[1]);	
	if(!visible) return;

	/* setup labels */
	makeCargoLabels(planet, NULL);	/* setup text for cargo amounts */
/* draw the background */
	glTranslatef(0.0,0.0,-0.1);
    abox.draw ( a, b, PUSTYLE_SMALL_BEVELLED, colour, isReturnDefault() ) ;
	glTranslatef(0.0,0.0,0.1);
	vector<int> display_list;
	display_list.clear();
	for(int j=0;j<BUILDING_NUM;j++)
	{
//		printf("can build: %d\n",planet->canBuild(0,j));
//		printf("is build: %d\n",planet->isBuilding(0,j));
//		printf("has build: %d\n\n",planet->building[j]);

/* it updates the building list every frame
	I should setup some sort of callback or event so that this is done only
	when the planet finishes building an item
	Also, isBuilding seems broken right now  */

		if( planet->canBuild(BUILDING,j) &&
			!planet->isBuilding(BUILDING,j) && !planet->building[j] )
		{
		display_list.push_back(j);
		}
	}
    buildChoice->setDisplayList(display_list);


/* same as last comment, for ships here */
	display_list.clear();
    for(int j=0;j<SHIP_NUM;j++)
	{
	if( planet->canBuild(SHIP,j) )			
		{
		display_list.push_back(j);
		}
	}
    shipChoice->setDisplayList(display_list);

/* display the building queue */
	buildQueue->setDisplayList( makeQueueList() );

/* the previous three sections should be done with a callback or event, not
	per frame! */


/* some tmp stuff, should move it to a display list */
	int progress = 0;
	glTranslatef(abox.min[0], abox.min[1], 0.1);
	int start_x, start_y, end_x, end_y;
	start_x = (int) 96*size_mult;
	start_y = (int) 140*size_mult;
	end_x = (int) 193*size_mult;
	end_y = (int) 160*size_mult;

	if(planet != NULL)
	{
		progress = (int) (end_x * planet->getProgress());
	}

	glBegin(GL_QUADS);

	glColor3f(1.0,0.2,0.0);
	glVertex3i(0,start_y,0);
	glColor3f(0.8,0.8,0.2);
	glVertex3i(start_x,start_y,0);
	glVertex3i(start_x,end_y,0);
	glColor3f(1.0,0.2,0.0);
	glVertex3i(0,end_y,0);

	glColor3f(0.8,0.8,0.2);
	glVertex3i(start_x,start_y,0);
	glColor3f(0.0,1.0,0.2);
	glVertex3i(end_x,start_y,0);
	glVertex3i(end_x,end_y,0);
	glColor3f(0.8,0.8,0.2);
	glVertex3i(start_x,end_y,0);

	glColor3f(0.0,0.0,0.0);
	glVertex3i(progress,start_y,0);
	glVertex3i(end_x,start_y,0);
	glVertex3i(end_x,end_y,0);
	glVertex3i(progress,end_y,0);

	glEnd();

	glTranslatef(-abox.min[0], -abox.min[1], 0.1);
}

void PlanetView::setPlanet(SolarSystem *p)
{
	planet = p;
	planet_name->setLabel( planet->nameid );
}

void PlanetView::cargoClicked(int a)
{
	setbuilditem(CARGO,a,1);
}

void PlanetView::buildClicked(int a)
{
	setbuilditem(BUILDING, a,1);
}

void PlanetView::shipClicked(int a)
{
	setbuilditem(SHIP, a,1);
}

void PlanetView::buildQueueClicked(int a)
{
}

vector<int> PlanetView::makeQueueList()
{

	queue<ProdItem> prod_q;
	prod_q = planet->getProdQueue();      /* huge comp warning!! */
	int display_item = 0;
	vector<int> item_list;
	ProdItem prod_item;
	while( !prod_q.empty() )
	{
		prod_item = prod_q.front();
		switch(prod_item.group)
		{
		case(CARGO): display_item = prod_item.type;				
			break;
		case(BUILDING): display_item = prod_item.type + CARGO_NUM;
			break;
		case(SHIP): display_item = prod_item.type + CARGO_NUM + BUILDING_NUM;
			break;
		}
		item_list.push_back(display_item);
		prod_q.pop();
	}
	return item_list;
}

int PlanetView::checkHit( int button, int updown, int x, int y)
{
//	printf("planetview checkHit called: button: %d, status: %d,
	// x: %d, y: %d\n", button, updown, x, y);
//	printf("action box: %d %d %d %d\n", abox.min[0], abox.min[1],
//	abox.max[0], abox.max[1]);
	if((x > abox.min[0]) && (x < abox.max[0]) && (y > abox.min[1])
		&& (y < abox.max[1]) ) printf("inside planetview\n");
	else return 0;
	// translate the x, y to where this widget is
	x -= abox.min[0];
	y -= abox.min[1];
//  printf("planetview checkHit called: button: %d, status: %d, x:
//	%d, y: %d\n", button, updown, x, y);
	return 0;
};
