/***************************************************************************
                          network.h  -  description
                             -------------------
    begin                : Mon May 8 2000
    copyright            : (C) 2000 by Blake Friesen
    email                : umfrie64@cc.umanitoba.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef NETWORK_H
#define NETWORK_H

#include "unit_def.h"
/*
Protocol definitions
*/

struct proto_header
{
	char type;
	char funct;
};

#define PROTOCOL_VERSION		0.1
#define MAX_CLIENTS				16
#define MAX_PACKET_SIZE			2048
#define MAX_CHAT_SIZE			80
#define MAX_NAME_SIZE			16
/*
general game definitions
*/
#define GAME_UPDATE			 	0	
#define GAME_JOIN				1   /* player sends this to join */
#define GAME_START				2	/* server sends this to start game */
#define GAME_END				4	/* game over */
#define CHAT_MSG				8 	/* player sends chat msg */

struct game_join
{
	int player_id;		/* player sends -1 on join, gets packet back 
				   with a proper ID */
	char player_name[MAX_NAME_SIZE];
};

struct chat_msg
{
	int from_player;
	int to_player;	/* -1 for everyone, else int for someone specific */
	char msg[MAX_CHAT_SIZE];
};

struct game_start
{
	int my_id;
	int my_start_planet;
	int num_players;
	char names[MAX_CLIENTS][MAX_NAME_SIZE];
};

/*
Planet definitions
*/
#define PLANET_UPDATE			1	
#define PRODUCTION_DONE			1		/* planet prod done */
#define PRODUCTION_REQUEST		2		/* player requests new production or granted  */
#define PLANET_UPDATE_PART		4		/* updates cargo, buildings and current prod item */
#define PLANET_UPDATE_BUILDING		8
#define PLANET_UPDATE_POSITION		16
#define PLANET_UPDATE_OWNER		32
#define PLANET_UPDATE_INFO		64

struct production_request
{
	int planet, group, type, quantity;
	int current_item; /* unique id of item */
};

struct production_done
{
	int planet;
};

struct update_part
{
	int planet, cargo[CARGO_NUM];
	bool building[BUILDING_NUM]; /* info for planet*/
	int current_item; 
	/* item that _should_ be at front of production queue */
	float progress;
};

struct update_building
{
	bool planet, building[BUILDING_NUM];
};

struct update_owner
{
	int planet, new_owner;
};

/*
Fleet definitions
*/
#define FLEET_UPDATE			2

#define FLEET_ADD				1 /* adds new fleet, probably same as update */
#define FLEET_ADD_WAYPOINT		2 /* adds waypoint */
#define FLEET_UPDATE_DEST		3 /* adds destination */
#define FLEET_UPDATE_CARGO		4
#define FLEET_UPDATE_SHIPS		5
#define FLEET_UPDATE_POSITION		6
#define FLEET_TRANSFER_CARGO		7


#define MAX_WAYPOINTS 			32

/* 	send to add either a direct waypoint or a destination for a fleet,
	or to update all waypoints for a fleet. */
struct fleet_waypoint
{
	int fleet_id;
	char waypoints[MAX_WAYPOINTS];
};

struct update_cargo
{
	int fleet_id, cargo[CARGO_NUM];
};

struct fleet_transfer_cargo
{
	int source, dest;
	char isfleet; /* dest is a fleet or not */
	int cargo[CARGO_NUM]; /* amount to transfer */
};

#endif
