/***************************************************************************
                          foundationserver.cpp  -  description
                             -------------------
    begin                : Mon Jan 31 2000
    copyright            : (C) 2000 by Blake Friesen
    email                : umfrie64@cc.umanitoba.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "foundationserver.h"

FoundationServer::FoundationServer()
{
	ne.bindsocket(5050);
	started = 0;
	loadnames();
	SolarSystem::loadcosts();
	player_num = 0;
}

FoundationServer::~FoundationServer()
{
}

void FoundationServer::wait_for_start()
{
	while(!started) net_wait();
}

void FoundationServer::game_loop()
{
/* production/growth cycle */
	int ship_type, fleet_id;
	Fleet *updated_fleet;
	while(1)
	{	
	printf("game loop\n");
	moveFleet();
	for(int i=0;i<PLANET_NUM;i++)
	{
		planet[i]->grow();
		if ( (ship_type = planet[i]->produce())  != -1)
		{
			fleet_id = addShip( ship_type, planet[i]->pos, planet[i]->owner );
			updated_fleet = getFleet( fleet_id );
			sendFleet( player[planet[i]->owner].addr, updated_fleet, 1);
		}
	}
	sleep(1);
	}
}/**  */


void FoundationServer::setBuildItem(int p, int g, int t)
{
	planet[p]->setBuildItem(g,t);	
}
/** finds a player by their Inet address
searchs by IP and port
returns -1 on unknown addr */

int FoundationServer::find_player(struct sockaddr_in addr)
{
	printf("looking for player: %d %d\n", addr.sin_addr.s_addr, addr.sin_port);

	for(int i=0;i<player_num;i++)
	{
	printf("checking player %d: %d %d\n", i,
		player[i].addr.sin_addr.s_addr, player[i].addr.sin_port);
		if( player[i].addr.sin_addr.s_addr == addr.sin_addr.s_addr
		&& player[i].addr.sin_port == addr.sin_port )
			return i;
	}

	printf("unable to find player!");
/* this should be removed for the finished version */
	assert(0);
}

void FoundationServer::moveFleet()
{
	
	int dest_planet;
	int can_see, could_see;
	can_see = could_see = 0;
	dest_planet = 0;
	
	printf("moving all fleets...\n");
	for(int x = 0;x < fleet.size();x++ )
	{
		printf("moving fleet, id: %d\n", fleet[x]->getId() );
		if(fleet[x]->numDest() > 1)
		{
/* can we see the fleet from the planet it is headed towards before moving? */
		dest_planet = find_planet(fleet[x]->getDest(1) );
		could_see = canSee( dest_planet, fleet[x]->pos );
		if( fleet[x]->move() )
		{
			printf("fleet reached a planet\n");
		/* 	could_see = 0 because we haven't compared it against the new
			waypoint */
			could_see = 0;
		}
/* can we see the fleet from the planet it is headed towards after moving? */
		if(fleet[x]->numDest() == 1)
			dest_planet = find_planet(fleet[x]->getDest(0) );
		else
			dest_planet = find_planet(fleet[x]->getDest(1) );

		can_see = canSee( dest_planet, fleet[x]->pos );
		printf("can see: %d could see %d\n", can_see, could_see);
/* 	we'll send a "ADD WAYPOINT" to the non-owners, containing the next
	waypoint.  The client should then add it and display it properly
	or if it has no info on the fleet, request a full update for the fleet.
	The server should then send all allowable fleet info to that player
	ie. based on sensor technology, range, or political alliance */			
            if( can_see && (can_see != could_see))
				sendFleet( player[ planet[dest_planet]->owner].addr,
					fleet[x], could_see);
		}
	}
}



int FoundationServer::start_game()
{	
/* this should be broken up into a function for sending a planet to a player
and a fleet to a player */
/* must send planet info to all players */
	char *buf = (char *)malloc(MAX_PACKET_SIZE);
	int i = 0;
	for(i = 0; i < PLANET_NUM; i++)
	{
		int len = 0;
		memcpy(buf, &i, sizeof(i));	/* id of planet */
		len += sizeof(i);
		len += planet[i]->getRaw(buf+len);
		printf("sending planet info %d\n", i);
		for(int j=0; j<player_num; j++)
		{
			printf("send planet id: %d, size: %d\n", (int )*buf, len);
			ne.sendpacket( player[j].addr, PLANET_UPDATE, PLANET_UPDATE_INFO,
				buf, len);
			usleep(100);
		}
	}
	/*send the fleet info to the players */
	for(int j=0; j<player_num; j++)
	{
		vector<Fleet *> player_fleet;
		player_fleet = getFleetOwner(j);
		printf("sending %d fleets to %d\n", player_fleet.size(), j);
		for(i = 0;i < player_fleet.size();i++)
		{
		printf("sending fleet %d\n", i);
		int len = 0;
		memset(buf, 0, MAX_PACKET_SIZE);
		len = player_fleet[i]->getRaw(buf);
		ne.sendpacket( player[j].addr, FLEET_UPDATE, FLEET_ADD,
				buf, len);
		}
	}
/* tmp
*/
	for(i=0;i<fleet.size();i++)
		printf("fleet %d is owned by %d\n", i, fleet[i]->owner);

	free(buf);
	started = 1;
	return 1;
}

void FoundationServer::handle_game_join(sockaddr_in addr, char *buf, int len)
{
	/* initializes server side player info
	   does not actually check and make sure that the name is not taken.
		(which it should)
	*/
	struct game_join st_game_join;
	assert(len == sizeof(st_game_join));
	memcpy(&st_game_join, buf, sizeof(st_game_join));
	player[player_num].addr = addr;
	strcpy(player[player_num].name, st_game_join.player_name);
	st_game_join.player_id = player_num;
/*
	ne.sendpacket(addr, GAME_UPDATE, GAME_JOIN,
			(char *)&st_game_join, sizeof(st_game_join));
*/
	player_num++;
}

void FoundationServer::handle_chat_msg(char *buf, int len)
{
	printf("server got chat msg.\n");
	for(int i = 0;i<player_num;i++)
	{
		ne.sendpacket(player[i].addr, GAME_UPDATE, CHAT_MSG,
			buf, len);
	}
}

void FoundationServer::handle_game_start(sockaddr_in sock, char *buf, int len)
{
	send_player_info();
	start_game();
}

void FoundationServer::handle_planet_info(char *buf, int len)
{
}

void FoundationServer::handle_add_waypoint(sockaddr_in addr, char *buf, int len)
{
/*	We've been notified that a user added a waypoint for a fleet.
	TODO:
	-We should be checking the addr for security.
		(use findplayer)
*/
   	struct fleet_waypoint st_fw;
	memcpy(&st_fw, buf, sizeof(fleet_waypoint) );

	Fleet *f;
	if( (f = getFleet(st_fw.fleet_id) ) != NULL)
	{
		printf("server adding waypoint\n");
		f->setDest( planet[ st_fw.waypoints[0] ]->getPos() );
	}
	else
	{
		printf("*** unknown fleet ***");
		/* print out extra info like source player and fleet id */
	}

}



/** seeds a planet with a player when
game starts */
int FoundationServer::init_planet(int player_id)
{
	int start_planet = random() % PLANET_NUM;
	while(planet[ start_planet ]->owner != -1)
		start_planet = random() % PLANET_NUM;
	planet[start_planet]->owner = player_id;
	planet[start_planet]->setCargo(1000000, CIVILIAN);
	planet[start_planet]->setCargo(50, FACTORY);
	planet[start_planet]->setCargo(50, MINE);
	planet[start_planet]->setCargo(5000, TRI);
	planet[start_planet]->setCargo(2000, DEU);
	planet[start_planet]->owner = player_id;

/* this creates an initial fleet for the player */
	Point p = planet[start_planet]->getPos();
	addShip(1, p, player_id);
	addShip(1, p, player_id);
	addShip(0, p, player_id);
	printf("done initing planet %d for %d\n", start_planet, player_id);
	return start_planet;
}


/** sends player names to everyone on
game start along with their starting location
and playerid */
void FoundationServer::send_player_info()
{
	printf("sending player info...\n");
    struct game_start st_game_start;
	memset(st_game_start.names, 0, MAX_CLIENTS * MAX_NAME_SIZE);
	st_game_start.num_players = num_players;
	int len = 0;
	for(int i=0;i<MAX_CLIENTS;i++)
	{
		strcpy(st_game_start.names[i], player[i].name);
	}
	for(int i=0;i<player_num;i++)
	{
		st_game_start.my_id = i;
		st_game_start.my_start_planet = init_planet(i);
		ne.sendpacket(player[i].addr, GAME_UPDATE, GAME_START,
			(char *)&st_game_start, sizeof(st_game_start) );
	}
	printf("done sending player info...\n");
}
/**  */
void FoundationServer::handle_planet_part(sockaddr_in addr, char *buf, int len)
{
	int planet_id;
/* a bit of an abuse of len */
	len = 0;
	memcpy(&planet_id, buf, sizeof(planet_id));
/*	printf("answering request for planet update on %d\n"); */
	struct update_part st_planet_part;
	st_planet_part.planet = planet_id;
	st_planet_part.current_item = planet[planet_id]->getFrontId();
	st_planet_part.progress = planet[planet_id]->getProgress();	
	memcpy(st_planet_part.cargo, planet[ planet_id ]->cargo,
		sizeof(int)*CARGO_NUM);
	
	len += sizeof(st_planet_part);

	memcpy(st_planet_part.building, planet[ planet_id ]->building,
		sizeof(bool)*BUILDING_NUM);
	
	ne.sendpacket(addr, PLANET_UPDATE, PLANET_UPDATE_PART,
		(char *)&st_planet_part, len);
}

void FoundationServer::handle_transfer_cargo(sockaddr_in addr, char *buf,
						int len)
{
	struct fleet_transfer_cargo cargo_request;
	Platform *source, *dest;

	memcpy(&cargo_request, buf, sizeof(fleet_transfer_cargo) );
	printf("cargo transfer\nsource: %d dest: %d\n\n", cargo_request.source,
			cargo_request.dest);
	printf("is fleet: %d\n", cargo_request.isfleet);


	source = getFleet(cargo_request.source);
	if (cargo_request.isfleet)
	{
		dest = getFleet(cargo_request.dest);
	}
	else
	{
		dest = planet[cargo_request.dest];
	}
	printf("source: %d, dest: %d\n", source, dest);
	if ( (dest->owner != source->owner) && (dest->owner != -1) ) return;

	for(int i = 0;i < CARGO_NUM;i++)
	{
		xfer(source, dest, cargo_request.cargo[i], i);
	}

	sendFleet( player[source->owner].addr, (Fleet *)source, 1);	
  /* ok here */

	if ( cargo_request.isfleet )
	{
		printf("** dest is fleet!! **\n\n");
		sendFleet( player[dest->owner].addr, (Fleet *)dest, 1);
	}
	else
	{
/* 	this must look somewhat confusing
	if the destination is a fleet, send the updated data
	if it's not a fleet, ie a solarsystem/planet, we must check and possibly
	change ownership
*/
		if(dest->owner == -1)
		{
			dest->owner = source->owner;
//			sendPlanet(player[dest->owner].addr, (SolarSystem *)dest, 1);
//	tmp
			int i = dest->getId();
			int len = 0;
			char *buf = ( char *) malloc(MAX_PACKET_SIZE);
			memcpy(buf, &i, sizeof(i));	/* id of planet */
			len += sizeof(i);
			len += planet[i]->getRaw(buf+len);
			printf("sending planet info %d\n", i);
			printf("send planet id: %d, size: %d\n", (int )*buf, len);
			ne.sendpacket( player[ dest->owner].addr, PLANET_UPDATE,
				PLANET_UPDATE_INFO,	buf, len);


		}
	}
}

void FoundationServer::handle_prod_request(sockaddr_in addr, char *buf, int len)
{
	struct production_request st_prod;
	memcpy(&st_prod, buf, sizeof(st_prod));

	st_prod.current_item =
		planet[ st_prod.planet ]->setBuildItem(st_prod.group, st_prod.type);
	printf("build request: %d %d\n", st_prod.group, st_prod.type);
	/* 	we couldn't add the production item so we're not notifying player
		of a new addition */	
	if(!st_prod.current_item)
	{
		printf("production request denied\n");
		return;
	}

	int i = find_player(addr);
	/* now send it back if it's accepted */
	ne.sendpacket(player[i].addr, PLANET_UPDATE, PRODUCTION_REQUEST,
		(char *)&st_prod, len);
	printf("setting prod for planet: %d\n", st_prod.planet);
}
