/***************************************************************************
                          system.cpp  -  description
                             -------------------
    begin                : Sat Jan 1 2000
    copyright            : (C) 2000 by Blake Friesen
    email                : umfrie64@cc.umanitoba.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <system.h>
#include <stdio.h>
#include <sys/time.h>
#include <unistd.h>
#include <math.h>
#include <fstream.h>
#include <string>
#include "unit_def.h"

#define CONST_FACT 25000

ShipInfo *SolarSystem::shipdata=NULL;
ProdItem SolarSystem::b_template[G_NUM][T_NUM];
int SolarSystem::unique_id = 5; /* start at 5... */

void SolarSystem::loadcosts(){
	ifstream input("prodcosts.txt");
	for(int x=0;x<G_NUM;x++)
	for(int y=0;y<T_NUM;y++){
		int g, t, tri_cost, pre;
		string s;
		input >> g >> t >> tri_cost >> pre;
		input >> s;
		b_template[g][t].setItem(tri_cost,g,t,pre);
	}
}

SolarSystem::SolarSystem(){
	owner = -1;
	strcpy(nameid, "planet");
	bzero(building, BUILDING_NUM);
	building[0] = 1;
	cout << "building: " << building[0];
	/* resource density calculation
	(cos(x-skew)+1)/concentration;
	 larger skew, more to the right, hence richer
	 larger concentration, richer.  concentration: 0 < c < 1 */
	int l=5000;

	while( l > 480)
	{
		pos.x=(int)(750.0*rand()/(RAND_MAX+1.0))-375;
		pos.y=(int)(750.0*rand()/(RAND_MAX+1.0))-375;
		pos.z=(int)(750.0*rand()/(RAND_MAX+1.0))-375;
		l = (int) sqrt( pow(pos.x,2) + pow(pos.y,2) + pow(pos.z,2));
	}
//	cout << "l: " << l << endl;
	timeval time_seed;
	gettimeofday(&time_seed, NULL);
	srandom(time_seed.tv_usec );
	int tri_skew = (int)(20.0*rand()/(RAND_MAX+1.0));
	int deu_skew = 0;
	double tri_concentration= sqrt(rand()/(RAND_MAX+1.0));
//	printf("random: %f\n", tri_concentration); 
	if(tri_concentration < 0.1) tri_concentration = 0.1;
	float deu_concentration=1;
	
	for(int x=0;x<50;x++){
		int tri_den=0, deu_den=0;
		float tri_r = ((x-25-tri_skew)*7.2);
		float deu_r = ((x-25-deu_skew)*7.2);		
		
		if ( (tri_r >= -180) &&(tri_r <=180) )
			tri_den = (int)((cos(tri_r/57.3)+1)/tri_concentration*CONST_FACT);

		if ( (deu_r>= -180) &&(deu_r<=180) )
			deu_den = (int)((cos(deu_r/57.3)+1)/deu_concentration*CONST_FACT);
//		cout << tri_den << endl;
//		cout << tri_r << " " << tri_den << endl;		
		tri_density[x]=tri_den;	
		deu_density[x]=deu_den;
		x_range[x]=x;			
		}
	
}

void SolarSystem::grow(){
/*
 cout << "\ncurrentItem: " << currentItem.empty() << "\t" <<
                prod_q.empty() << endl;
	cout << "grow\n";
*/
	int x=DENSITY_SIZE;
	while((x>0) &&(tri_density[x]==0))x--;
//	int tri_mined = (int)(cargo[CIVILIAN]*0.007)/(DENSITY_SIZE-x+1)*50;
	int tri_mined = (int)(cargo[MINE]*10)/(DENSITY_SIZE-x+1)*50;
	//cout << "pop: " << cargo[CIVILIAN] << endl;
	if(tri_mined<tri_density[x])
	{
		tri_density[x] -= tri_mined;
		cargo[TRI] += tri_mined;	
	}
	else
	{
		tri_mined=0;
		cargo[TRI] += (int)(tri_density[x]);
		tri_density[x]=0;
	}

	int growth = (int) (cargo[CIVILIAN] *0.0005);
	if(cargo[TRI]>growth)
	{
		cargo[TRI] -= growth;
		cargo[CIVILIAN]+=growth;
		return;
	}
	else
	{
		cargo[CIVILIAN]+=cargo[TRI];
		cargo[TRI] = 0;
	}
}

int SolarSystem::produce(){
	int update = -1;	
	int prod_amount = 0;
	// now do production
/*
	if(currentItem.empty() && !prod_q.empty()){
//		cout << "setting current item\n";
		currentItem = prod_q.front();
		prod_q.pop();
		}
	currentItem.progress(cargo[FACTORY]);
*/
	if( !prod_q.empty() )
	{
		if (cargo[FACTORY] < cargo[TRI]) prod_amount = cargo[FACTORY];
		else prod_amount = cargo[TRI];
		if (prod_amount < 0) prod_amount = 0;
		prod_q.front().progress(cargo[FACTORY]);
		printf("front tri: %d\n", prod_q.front().tri);
	//	cout << "currentItem: " << currentItem.empty() << "\t" <<
	//		currentItem.type << endl;
		if(prod_q.front().done())
		{
			int g = prod_q.front().group;
			int t = prod_q.front().type;
			cout << "finished\n" << g << " " << t << endl;
	/* it's a cargo item of some type */
			if(g == CARGO) cargo[t]++;
	
	/* 	we're adding a new ship with the same ownership as this planet,
	    which should be a correct assumption */
			if(g == SHIP)
				{
//					addShip(t, pos, owner);
					update = t;
				}
	/* this is a 'unique' building */
			if(g == BUILDING) building[t]=1;
			prod_q.pop();
			/* currentItem.empty() should now be true */
		}
	}
	return update;
}


vector <int> SolarSystem::getneeds(){
	vector<int> needs;
	needs.resize(2);
	needs[0] = int(cargo[CIVILIAN] * 0.05);
	needs[1] = int(cargo[CIVILIAN] * 0.08);
	return needs;
}

int SolarSystem::setBuildItem(int g, int t){
	if(canBuild(g, t))
	{
		ProdItem p = b_template[g][t];
		p.id = unique_id++;
		prod_q.push(p);
//	printf("setting build item group: %d type: %d\n", p.g, p.t);
		return unique_id;
	/* return the id of the item we just added */
	}
/* can't add, return 0 */
	return 0;
}

int SolarSystem::setBuildItem(int g, int t, int id){
	ProdItem p = b_template[g][t];
	p.id = id;
	prod_q.push(p);
//	printf("setting build item group: %d type: %d\n", p.g, p.t);
/* can't add, return 0 */
	return 0;
}


bool SolarSystem::canBuild(int g, int t){
//	cout << "\ncan build " << g << " " << t <<
//	building[b_template[g][t].prereq];
	return building[b_template[g][t].prereq];
}

bool SolarSystem::isBuilding(int g, int t){
	// checks if an item of type t is in the queue
//	printf("is building called\n");
	queue<ProdItem> temp_q = prod_q;
	while(!temp_q.empty()){
		ProdItem p = temp_q.front();
//		printf("checking temp_q: %d\n", p.type);
		if( (p.group == g) && (p.type == t) ) return 1;
		temp_q.pop();
		}
	return 0;
}

bool SolarSystem::isFleet(){
	return 0;
}

bool SolarSystem::checkOwner(int c){
//	cout << "system checking owner\n";
/* this needs a major fix........ */
	if(owner == OWNER) return 1;
	if((owner == -1) && (c == CIVILIAN)){
//		cout << "claiming planet!!\n";
		owner = OWNER;
		return 1;
		}
	return 0;
}

/*
void SolarSystem::addShip(){
	cout << "should not call me!\n";
}

*//**  */

queue<ProdItem> SolarSystem::getProdQueue()
{
	return prod_q;
}

int SolarSystem::getFrontId()
{
	if( prod_q.empty() ) return 0;
	return prod_q.front().id;
}

int SolarSystem::setFrontId(int id)
{
	/* pops the queue until the proper item is at the front of the queue
		returns 0 if it's not in the queue at all, which means a full planet
		update is required since the info is not properly synchronized
		if id is 0, that means we must empty the queue
	*/

	printf("setting production id to: %d\n", id);
	if(id == 0)
	{
		while(!prod_q.empty() ) prod_q.pop();
		return 1;
	}

	while(!prod_q.empty())
	{
		if(id == prod_q.front().id) return id;
		prod_q.pop();
	}
	printf("*** planet data out of sync ***\n");
	return 0;
}

int SolarSystem::getRaw(char *buf)
{
	int offset = 0;
	int prod_size = 0;
	offset += Platform::getRaw(buf);
	queue<ProdItem> trans_q = prod_q;
	prod_size = prod_q.size();
	memcpy(buf+offset, &prod_size, sizeof(prod_size));
	offset += sizeof(prod_size);
	for(int i = 0;i<(signed int)trans_q.size();i++)
	{
		ProdItem p = trans_q.front();
		trans_q.pop();
		memcpy(buf+offset, &p, sizeof(p));
		offset += sizeof(p);
	}
	printf("offset %d\n", offset);
//	memcpy(buf+offset, &currentItem, sizeof(currentItem));
//	offset += sizeof(currentItem);
	memcpy(buf+offset, &popmax, sizeof(popmax));
	offset += sizeof(popmax);

	memcpy(buf+offset, &tri_density, sizeof(float) * DENSITY_SIZE);
	offset += (sizeof(float) * DENSITY_SIZE);
	memcpy(buf+offset, &deu_density, sizeof(float) * DENSITY_SIZE);
	offset += (sizeof(float) * DENSITY_SIZE);
	memcpy(buf+offset, &x_range, sizeof(float) * DENSITY_SIZE);
	offset += (sizeof(float) * DENSITY_SIZE);

	return offset;
}
/**  */
int SolarSystem::setRaw(char *buf)
{
	int offset = 0;
	int prod_size = 0;
	offset += Platform::setRaw(buf);
	while(!prod_q.empty()) prod_q.pop();
	memcpy(&prod_size, buf+offset, sizeof(prod_size));
	offset += sizeof(prod_size);
/*
	printf("size of prod_q: %d\n\n", prod_size);

	for(int i = 0;i<prod_size;i++)
	{
		ProdItem p;
		memcpy(&p, buf+offset, sizeof(p));
		offset += sizeof(p);
		prod_q.push(p);
	}
	memcpy(buf+offset, &currentItem, sizeof(currentItem));
	offset += sizeof(currentItem);
	memcpy(buf+offset, &popmax, sizeof(popmax));
	offset += sizeof(popmax);

	memcpy(&tri_density, buf+offset, sizeof(float) * DENSITY_SIZE);
	offset += (sizeof(float) * DENSITY_SIZE);
	memcpy(&deu_density, buf+offset, sizeof(float) * DENSITY_SIZE);
	offset += (sizeof(float) * DENSITY_SIZE);
	memcpy(&x_range, buf+offset, sizeof(float) * DENSITY_SIZE);
	offset += (sizeof(float) * DENSITY_SIZE);
*/
	return offset;
}
/** returns the percentage complete of the top
production item */
float SolarSystem::getProgress()
{
	float progress;
	char group, type;
	if ( prod_q.empty() ) return 0;
	group = prod_q.front().group;
	type = prod_q.front().type;
	progress = ((float)prod_q.front().tri) /
		((float)b_template[group][type].tri);
	printf("progress on planet: %f\n", progress);
	return progress;
}
/** synchronizes building progress of the
first item on a planet */
void SolarSystem::setProgress(float progress)
{
	char group, type;
	if ( prod_q.empty() ) return;
	group = prod_q.front().group;
	type = prod_q.front().type;
	prod_q.front().tri = b_template[group][type].tri -
		(int)(b_template[group][type].tri * progress);
}
